#!/usr/bin/env python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gtk

import logging
_logger = logging.getLogger('ourstoriesui')
_logger.setLevel(logging.DEBUG)

from gettext import gettext as _
from sugar.graphics.icon import Icon
from sugar.graphics.toolcombobox import ToolComboBox
from sugar.graphics.combobox import ComboBox
from sugar.graphics.menuitem import MenuItem
from sugar.graphics.toolbutton import ToolButton
from sugar.graphics import style
from sugar.activity import activity

from storymanager import StoryManager

import hippo
import gobject

def toolize_widget(widget, expand=False):
    tool_item = gtk.ToolItem()
    tool_item.set_expand(expand)
    tool_item.add(widget)
    return tool_item

def align_widget(widget, x, y, xscale, yscale):
    align = gtk.Alignment(x, y, xscale, yscale)
    align.add(widget)
    return align

class BrowseToolbar(gtk.Toolbar):
    __gtype_name__ = 'BrowseToolbar'

    def __init__(self):
        gtk.Toolbar.__init__(self)
        self._combo = ComboBox()
        self._combo.append_item(0, _('My Stories'))
        self._combo.append_item(1, _('Our Stories'))
        self._combo.set_active(0)
        self._AddSeperator()
        self.insert(ToolComboBox(combo=self._combo), -1)
        self._AddSeperator()

    def _AddSeperator(self):
        seperator = gtk.SeparatorToolItem()
        seperator.set_draw(False)
        seperator.set_expand(True)
        self.insert(seperator, -1)

class RecordToolbar(gtk.Toolbar):
    __gtype_name__ = 'RecordToolbar'

    def __init__(self):
        gtk.Toolbar.__init__(self)

        self._storymanager = StoryManager()
        self._storymanager.connect('record_expired', self._record_expired_cb)
        self._storymanager.connect('record_ticker', self._record_ticker_cb)

        self._ispause = False;
        self._record = ToolButton('media-record')
        self._record.set_tooltip(_('Start Recording'))
        self._record.connect('clicked', self._record_cb)
        self.insert(self._record, -1)
        
       
        self._progress = gtk.ProgressBar()
        self.insert(toolize_widget(align_widget(self._progress, 0.5, 0.5, 1, 0), expand=True), -1)

        self._timeelapsed = gtk.Label()
        self._timeelapsed.set_markup('<span size="xx-large">0:00</span>')
        self.insert(toolize_widget(self._timeelapsed), -1)
        
   
    def _record_cb(self, widget):
        if (self._ispause):
          self._do_stop_record()
        else:
          self._do_start_record()

    def _do_start_record(self):
      self._record.set_icon('media-playback-stop')
      self._record.set_tooltip(_('Stop Recording'))
      self._ispause = True
      self._storymanager.startRecord()

    def _do_stop_record(self):
      self._record.set_icon('media-record')
      self._record.set_tooltip(_('Start Recording'))
      self._ispause = False
      self._storymanager.stopRecord()
      # reset ticker and progress bar
      self._timeelapsed.set_markup('<span size="xx-large">0:00</span>')
      self._progress.set_fraction(0)
     

    def _record_expired_cb(self, message):
      self._do_stop_record()

    def _record_ticker_cb(self, message, tuple):
      progress, total = tuple

      min = str(progress/60)
      sec = str(progress%60)
      if progress%60 < 10:
        sec = '0'+sec
      self._timeelapsed.set_markup('<span size="xx-large">' + min + ':' + sec + '</span>')
      percentage_progress = float(progress)/float(total)
      self._progress.set_fraction(percentage_progress)
   
        
class ListenToolbar(gtk.Toolbar):
    __gtype_name__ = 'ListenToolbar'

    def __init__(self):
        gtk.Toolbar.__init__(self)

        self._ispause = False;
        self._play = ToolButton('media-playback-start')
        self._play.set_tooltip(_('Play'))
        self._play.connect('clicked', self._play_cb)
        self.insert(self._play, -1)
        
        self._timeelapsed = gtk.Label()
        self._timeelapsed.set_markup('<span size="xx-large">0:30</span>')
        self.insert(toolize_widget(self._timeelapsed), -1)
        
        
        self._progress = gtk.ProgressBar()
        self._progress.set_fraction(0.5)
        self.insert(toolize_widget(align_widget(self._progress, 0.5, 0.5, 1, 0), expand=True), -1)
        
        self._totaltime = gtk.Label()
        self._totaltime.set_markup('<span size="xx-large">1:00</span>')
        self.insert(toolize_widget(self._totaltime), -1)
        
        self._stop = ToolButton('media-playback-stop')
        self._stop.set_tooltip(_('Stop'))
        self._stop.connect('clicked', self._stop_cb)
        self.insert(self._stop, -1)
        
    def _stop_cb(self, widget):
        _logger.debug('Clicked Stop!')
    
    def _play_cb(self, widget):
        if (self._ispause):
          self._play.set_icon('media-playback-start')
          self._play.set_tooltip(_('Play'))
          self._ispause = False
        else:
          self._play.set_icon('media-playback-pause')
          self._play.set_tooltip(_('Pause'))
          self._ispause = True

