#!/usr/bin/env python

import logging
_logger = logging.getLogger('StoryManager')
_logger.setLevel(logging.DEBUG)

import os
import gobject
from datetime import datetime
from voice import VoiceRecorder

from xml.dom.minidom import Document
from xml.dom.minidom import parse

DEFAULT_OURSTORIES_PATH='/home/olpc/ourstories'
ISOFORMAT_DELIM=' '

class OurStoriesObject:
  """
  """

  def __init__(self, metafile=None, storyname=None, oggfile=None, datetime=None):
    # members
    self._storyname = storyname
    self._oggfile = oggfile
    self._datetime = datetime
    self._listening = False

    # deprecated
    if metafile:
      self._import_from_meta_file(metafile)

  def set_listening(self, status):
    self._listening = status

  def is_listening(self):
    return self._listening

  def _import_from_meta_file(self, metafile):
    meta_file = file(metafile)
    # parse
    filename = meta_file.readline().strip('\n')
    self._oggfile = filename
    self._storyname = filename.strip('.ogg')

  def get_storyname(self):
    return self._storyname

  def get_datestring(self):
    return self._datetime.isoformat(ISOFORMAT_DELIM).split(ISOFORMAT_DELIM)[0]

  def get_oggfile(self):
    return self._oggfile

  def xml_import(self, filepath):
    xmlfile = file(filepath, 'r')
    doc = parse(xmlfile)
    xmlfile.close()

    os = doc.getElementsByTagName('ourstories')

    for el in os[0].childNodes:
      if el.nodeType != 3 and el.tagName == 'story':
        self._storyname = el.getAttribute('title')
        date_format = '%Y-%m-%d' + ISOFORMAT_DELIM + '%H:%M:%S'
        self._datetime = datetime.strptime( el.getAttribute('date'), date_format)
        self._oggfile = el.getAttribute('oggfile')

    doc.unlink()

  def xml_export(self, filepath):
    doc = Document()
    os_block = doc.createElement('ourstories')
    os_block.setAttribute('version', str(0.1)) # 10-27-2007
    doc.appendChild(os_block)

    story = doc.createElement('story')
    story.setAttribute('title', self._storyname)
    story.setAttribute('date', self._datetime.isoformat(ISOFORMAT_DELIM))
    story.setAttribute('oggfile', self._oggfile)
    os_block.appendChild(story)

    # XXX check filepath successfulness

    xmlfile = file(filepath + '.xml', 'w')
    xmlfile.write(doc.toprettyxml(indent='  '))
    xmlfile.close()

    doc.unlink()


  # for sorting, compare by date
  def __cmp__(self, other):
    if self._datetime < other._datetime:
      return -1
    elif self._datetime > other._datetime:
      return 1
    else:
      return 0

class StoryManager:
  """ A singleton class """
  
  class __impl(gobject.GObject):
    """ implementation of the singleton interface """

    __gtype_name__ = 'StoryNamager'
    __gsignals__ = {
      'new_story_sig': (gobject.SIGNAL_RUN_FIRST,
                        gobject.TYPE_NONE,
                        ([])),
      'stop_listen':   (gobject.SIGNAL_RUN_FIRST,
                        gobject.TYPE_NONE,
                        ([object])),
      'record_ticker': (gobject.SIGNAL_RUN_FIRST,
                        gobject.TYPE_NONE,
                        ([object])),
      'record_expired':(gobject.SIGNAL_RUN_FIRST,
                        gobject.TYPE_NONE,
                        ([]))
    }

    def __init__(self, scan_dir):
      gobject.GObject.__init__(self)
      self._vr = VoiceRecorder()
      self._vr.connect('listen_eos', self._listen_eos_cb)
      self._vr.connect('record_expired', self._record_expired_cb)
      self._vr.connect('record_ticker', self._record_ticker_cb)
      self._stories = []
      self._currently_listening = None

      # Check path existance, if not, mkdir
      if not os.path.isdir(DEFAULT_OURSTORIES_PATH):
        _logger.info(DEFAULT_OURSTORIES_PATH + ' does not exist, mkdir')
        # FIXME: handle os.mkdir exception?
        os.mkdir(DEFAULT_OURSTORIES_PATH)

      self._scan_story_dir(scan_dir)

    def _scan_story_dir(self, scan_dir):
      self._stories = []
      dir_files = os.listdir(scan_dir)
      for filename in dir_files:

        if filename.count('.xml'):
          storyobj = OurStoriesObject()
          storyobj.xml_import(os.path.join(scan_dir, filename))
          self._stories.append(storyobj)


# # deprecated format
#        if filename.count('.os.meta'):
#          storyobj = OurStoriesObject(os.path.join(scan_dir, filename))
#          self._stories.append(storyobj)

      self._stories.sort(reverse=True)

    def get_stories(self):
      return self._stories

    def set_scandir(scan_dir):
      self._scan_story_dir(scan_dir)
   
    def spam(self):
      """ test method to return singleton id """
      return id(self)

    def startRecord(self):
      datetime_obj = datetime.utcnow().replace(microsecond=0)
      filename = 'story_' + datetime_obj.isoformat(ISOFORMAT_DELIM).replace(':', '-')
      fullpath = os.path.join(DEFAULT_OURSTORIES_PATH, filename)

      self._vr.startVoiceRecord(fullpath + '.ogg')

      new_story = OurStoriesObject(oggfile=fullpath + '.ogg', storyname=filename, datetime=datetime_obj)
      new_story.xml_export(fullpath)


    def stopRecord(self):
      self._vr.stopVoiceRecord()
      self._scan_story_dir(DEFAULT_OURSTORIES_PATH)
      self.emit('new_story_sig')

    def startListen(self, storyobj):
      if self._currently_listening:
        self.emit('stop_listen', self._currently_listening)
        self.stopListen()
        
      self._currently_listening = storyobj.get_oggfile()
      self._vr.startVoicePlay(self._currently_listening)

    def stopListen(self):
      self._currently_listening = None
      self._vr.stopVoicePlay()

    # voice recorder signal handlers
    def _listen_eos_cb(self, message):
      self.emit('stop_listen', self._currently_listening)
      self.stopListen()

    def _record_expired_cb(self, message):
      # pass on to UI
      self.emit('record_expired')
 
    def _record_ticker_cb(self, message, tuple):
      self.emit('record_ticker', tuple)


  # storage for the instance reference
  __instance = None

  def __init__(self, scan_dir = DEFAULT_OURSTORIES_PATH):
    if StoryManager.__instance is None:
      StoryManager.__instance = StoryManager.__impl(scan_dir)

    # store instance reference as the only member
    self.__dict__['_StoryManager__instance'] = StoryManager.__instance

  # delegate access to implementation
  def __getattr__(self, attr):
    return getattr(self.__instance, attr)

  def __setattr__(self, attr, value):
    return setattr(self.__instance, attr, value)

